# This file contains code to simulate the single cell RNA seq data sets for use
# in the downstream computational experiments.

suppressMessages({
  library(SingleCellExperiment)
  library(scran)
  library(scater)
  library(splatter)
})

# Functions

#' Extract the indexes of DE genes from a splatter simulated
#' SingleCellExperiment object
#'
#' @param sce A SingleCellExperiment object create using by a splatter
#'   simulation.
#'
#' @return A list of the DE genes in each group.
#'
extract_de_inds <- function(sce) {
  stopifnot(is(sce, "SingleCellExperiment"))

  n_groups <- length(unique(colData(sce)$Group))
  col_names <- paste0("DEFacGroup", 1:n_groups)
  data <- rowData(sce)[, col_names]

  out <- lapply(data, function(x) which(x != 1))

  names(out) <- paste0("group_", 1:n_groups)
  out
}

# Code

# Get the passed arguments from the command line via Snakemake.
args <- R.utils::commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging
print(args)

# Get the simulation parameters.
sim_pars <- jsonlite::fromJSON(args$sim_pars)

set.seed(sim_pars$seed)

# Create the parameters for the splatter simulation.
params <- newSplatParams(
  group.prob = sim_pars$group.prob,
  de.facLoc = sim_pars$de.facLoc,
  batchCells = sim_pars$batchCells,
)

# Run the splatter simulation.
sim <- splatSimulate(params, method = "groups", verbose = FALSE)

# Get the indices of the DE genes.
de_inds <- extract_de_inds(sim)

# Construct the log normalized counts needed for downstream analysis.
# TODO: Should we use deconvolution or just the naive size factors?
clusters <- quickCluster(sim)
sim <- computeSumFactors(sim, clusters = clusters)
sim <- logNormCounts(sim)

# Label the columns with the true simulated groups.
colLabels(sim) <- colData(sim)$Group

# Add information about the locations of the DE genes in each group.
sim <- SingleCellExperiment(
  sim,
  metadata = list(de_inds = de_inds)
)

# For debugging purposes.
print(sim)

saveRDS(sim, args$sim)
